@@ -310,7 +310,7 @@ def calc_key(board, mirror=False):
     key = 0
 
     for color in chess.COLORS:
-        mirrored_color = color ^ 1 if mirror else color
+        mirrored_color = color ^ mirror
 
         for i in range(chess.pop_count(board.pawns & board.occupied_co[color])):
             key ^= chess.POLYGLOT_RANDOM_ARRAY[mirrored_color * 6 * 16 + 5 * 16 + i]
@@ -331,9 +331,7 @@ def calc_key(board, mirror=False):
 def calc_key_from_filename(filename, mirror=False):
     white, black = filename.split("v")
 
-    color = chess.WHITE
-    if mirror:
-        color ^= 1
+    color = chess.WHITE ^ mirror
 
     key = 0
 
@@ -341,7 +339,7 @@ def calc_key_from_filename(filename, mirror=False):
         for i in range(white.count(piece)):
             key ^= chess.POLYGLOT_RANDOM_ARRAY[color * 6 * 16 + piece_index * 16 + i]
 
-    color ^= 1
+    color = not color
 
     for piece_index, piece in enumerate(PCHR):
         for i in range(black.count(piece)):
@@ -847,12 +845,12 @@ class WdlTable(Table):
             self.pieces = {}
 
             self.factor = {}
-            self.factor[chess.WHITE] = [0, 0, 0, 0, 0, 0]
-            self.factor[chess.BLACK] = [0, 0, 0, 0, 0, 0]
+            self.factor[0] = [0, 0, 0, 0, 0, 0] # White
+            self.factor[1] = [0, 0, 0, 0, 0, 0] # Black
 
             self.norm = {}
-            self.norm[chess.WHITE] = [0 for _ in range(self.num)]
-            self.norm[chess.BLACK] = [0 for _ in range(self.num)]
+            self.norm[0] = [0 for _ in range(self.num)] # White
+            self.norm[1] = [0 for _ in range(self.num)] # Black
 
             # Used if there are pawns.
             self.files = [PawnFileData() for _ in range(4)]
@@ -871,32 +869,32 @@ class WdlTable(Table):
                 data_ptr += self.num + 1
                 data_ptr += data_ptr & 0x01
 
-                self.precomp[chess.WHITE] = self.setup_pairs(data_ptr, self.tb_size[0], 0, True)
+                self.precomp[0] = self.setup_pairs(data_ptr, self.tb_size[0], 0, True)
                 data_ptr = self._next
                 if split:
-                    self.precomp[chess.BLACK] = self.setup_pairs(data_ptr, self.tb_size[1], 3, True)
+                    self.precomp[1] = self.setup_pairs(data_ptr, self.tb_size[1], 3, True)
                     data_ptr = self._next
                 else:
-                    self.precomp[chess.BLACK] = None
+                    self.precomp[1] = None
 
-                self.precomp[chess.WHITE].indextable = data_ptr
+                self.precomp[0].indextable = data_ptr
                 data_ptr += self.size[0]
                 if split:
-                    self.precomp[chess.BLACK].indextable = data_ptr
+                    self.precomp[1].indextable = data_ptr
                     data_ptr += self.size[3]
 
-                self.precomp[chess.WHITE].sizetable = data_ptr
+                self.precomp[0].sizetable = data_ptr
                 data_ptr += self.size[1]
                 if split:
-                    self.precomp[chess.BLACK].sizetable = data_ptr
+                    self.precomp[1].sizetable = data_ptr
                     data_ptr += self.size[4]
 
                 data_ptr = (data_ptr + 0x3f) & ~0x3f
-                self.precomp[chess.WHITE].data = data_ptr
+                self.precomp[0].data = data_ptr
                 data_ptr += self.size[2]
                 if split:
                     data_ptr = (data_ptr + 0x3f) & ~0x3f
-                    self.precomp[chess.BLACK].data = data_ptr
+                    self.precomp[1].data = data_ptr
             else:
                 s = 1 + int(self.pawns[1] > 0)
                 for f in range(4):
@@ -905,67 +903,67 @@ class WdlTable(Table):
                 data_ptr += data_ptr & 0x01
 
                 for f in range(files):
-                    self.files[f].precomp[chess.WHITE] = self.setup_pairs(data_ptr, self.tb_size[2 * f], 6 * f, True)
+                    self.files[f].precomp[0] = self.setup_pairs(data_ptr, self.tb_size[2 * f], 6 * f, True)
                     data_ptr = self._next
                     if split:
-                        self.files[f].precomp[chess.BLACK] = self.setup_pairs(data_ptr, self.tb_size[2 * f + 1], 6 * f + 3, True)
+                        self.files[f].precomp[1] = self.setup_pairs(data_ptr, self.tb_size[2 * f + 1], 6 * f + 3, True)
                         data_ptr = self._next
                     else:
-                        self.files[f].precomp[chess.BLACK] = None
+                        self.files[f].precomp[1] = None
 
                 for f in range(files):
-                    self.files[f].precomp[chess.WHITE].indextable = data_ptr
+                    self.files[f].precomp[0].indextable = data_ptr
                     data_ptr += self.size[6 * f]
                     if split:
-                        self.files[f].precomp[chess.BLACK].indextable = data_ptr
+                        self.files[f].precomp[1].indextable = data_ptr
                         data_ptr += self.size[6 * f + 3]
 
                 for f in range(files):
-                    self.files[f].precomp[chess.WHITE].sizetable = data_ptr
+                    self.files[f].precomp[0].sizetable = data_ptr
                     data_ptr += self.size[6 * f + 1]
                     if split:
-                        self.files[f].precomp[chess.BLACK].sizetable = data_ptr
+                        self.files[f].precomp[1].sizetable = data_ptr
                         data_ptr += self.size[6 * f + 4]
 
                 for f in range(files):
                     data_ptr = (data_ptr + 0x3f) & ~0x3f
-                    self.files[f].precomp[chess.WHITE].data = data_ptr
+                    self.files[f].precomp[0].data = data_ptr
                     data_ptr += self.size[6 * f + 2]
                     if split:
                         data_ptr = (data_ptr + 0x3f) & ~0x3f
-                        self.files[f].precomp[chess.BLACK].data = data_ptr
+                        self.files[f].precomp[1].data = data_ptr
                         data_ptr += self.size[6 * f + 5]
 
             self.initialized = True
 
     def setup_pieces_pawn(self, p_data, p_tb_size, f):
-        j = 1 + int(self.pawns[chess.BLACK] > 0)
+        j = 1 + int(self.pawns[1] > 0)
         order = self.read_ubyte(p_data) & 0x0f
-        order2 = self.read_ubyte(p_data + 1) & 0x0f if self.pawns[chess.BLACK] else 0x0f
-        self.files[f].pieces[chess.WHITE] = [self.read_ubyte(p_data + i + j) & 0x0f for i in range(self.num)]
-        self.files[f].norm[chess.WHITE] = [0 for _ in range(self.num)]
-        self.set_norm_pawn(self.files[f].norm[chess.WHITE], self.files[f].pieces[chess.WHITE])
-        self.files[f].factor[chess.WHITE] = [0, 0, 0, 0, 0, 0]
-        self.tb_size[p_tb_size] = self.calc_factors_pawn(self.files[f].factor[chess.WHITE], order, order2, self.files[f].norm[chess.WHITE], f)
+        order2 = self.read_ubyte(p_data + 1) & 0x0f if self.pawns[1] else 0x0f
+        self.files[f].pieces[0] = [self.read_ubyte(p_data + i + j) & 0x0f for i in range(self.num)]
+        self.files[f].norm[0] = [0 for _ in range(self.num)]
+        self.set_norm_pawn(self.files[f].norm[0], self.files[f].pieces[0])
+        self.files[f].factor[0] = [0, 0, 0, 0, 0, 0]
+        self.tb_size[p_tb_size] = self.calc_factors_pawn(self.files[f].factor[0], order, order2, self.files[f].norm[0], f)
 
         order = self.read_ubyte(p_data) >> 4
         order2 = self.read_ubyte(p_data + 1) >> 4 if self.pawns[1] else 0x0f
-        self.files[f].pieces[chess.BLACK] = [self.read_ubyte(p_data + i + j) >> 4 for i in range(self.num)]
-        self.files[f].norm[chess.BLACK] = [0 for _ in range(self.num)]
-        self.set_norm_pawn(self.files[f].norm[chess.BLACK], self.files[f].pieces[chess.BLACK])
-        self.files[f].factor[chess.BLACK] = [0, 0, 0, 0, 0, 0]
-        self.tb_size[p_tb_size + 1] = self.calc_factors_pawn(self.files[f].factor[chess.BLACK], order, order2, self.files[f].norm[chess.BLACK], f)
+        self.files[f].pieces[1] = [self.read_ubyte(p_data + i + j) >> 4 for i in range(self.num)]
+        self.files[f].norm[1] = [0 for _ in range(self.num)]
+        self.set_norm_pawn(self.files[f].norm[1], self.files[f].pieces[1])
+        self.files[f].factor[1] = [0, 0, 0, 0, 0, 0]
+        self.tb_size[p_tb_size + 1] = self.calc_factors_pawn(self.files[f].factor[1], order, order2, self.files[f].norm[1], f)
 
     def setup_pieces_piece(self, p_data):
-        self.pieces[chess.WHITE] = [self.read_ubyte(p_data + i + 1) & 0x0f for i in range(self.num)]
+        self.pieces[0] = [self.read_ubyte(p_data + i + 1) & 0x0f for i in range(self.num)]
         order = self.read_ubyte(p_data) & 0x0f
-        self.set_norm_piece(self.norm[chess.WHITE], self.pieces[chess.WHITE])
-        self.tb_size[chess.WHITE] = self.calc_factors_piece(self.factor[chess.WHITE], order, self.norm[chess.WHITE])
+        self.set_norm_piece(self.norm[0], self.pieces[0])
+        self.tb_size[0] = self.calc_factors_piece(self.factor[0], order, self.norm[0])
 
-        self.pieces[chess.BLACK] = [self.read_ubyte(p_data + i + 1) >> 4 for i in range(self.num)]
+        self.pieces[1] = [self.read_ubyte(p_data + i + 1) >> 4 for i in range(self.num)]
         order = self.read_ubyte(p_data) >> 4
-        self.set_norm_piece(self.norm[chess.BLACK], self.pieces[chess.BLACK])
-        self.tb_size[chess.BLACK] = self.calc_factors_piece(self.factor[chess.BLACK], order, self.norm[chess.BLACK])
+        self.set_norm_piece(self.norm[1], self.pieces[1])
+        self.tb_size[1] = self.calc_factors_piece(self.factor[1], order, self.norm[1])
 
     def probe_wdl_table(self, board):
         self.init_table_wdl()
@@ -991,7 +989,7 @@ class WdlTable(Table):
             while i < self.num:
                 piece_type = self.pieces[bside][i] & 0x07
                 color = (self.pieces[bside][i] ^ cmirror) >> 3
-                bb = board.pieces_mask(piece_type, color)
+                bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
                 square = chess.bit_scan(bb)
                 while square != -1 and square is not None:
@@ -1007,7 +1005,7 @@ class WdlTable(Table):
             k = self.files[0].pieces[0][0] ^ cmirror
             color = k >> 3
             piece_type = k & 0x07
-            bb = board.pieces_mask(piece_type, color)
+            bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
             square = chess.bit_scan(bb)
             while square != -1 and square is not None:
@@ -1020,7 +1018,7 @@ class WdlTable(Table):
             while i < self.num:
                 color = (pc[i] ^ cmirror) >> 3
                 piece_type = pc[i] & 0x07
-                bb = board.pieces_mask(piece_type, color)
+                bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
                 square = chess.bit_scan(bb)
                 while square != -1 and square is not None:
@@ -1143,7 +1141,7 @@ class DtzTable(Table):
                 bside = int(board.turn != chess.WHITE)
         else:
             cmirror = 0 if board.turn == chess.WHITE else 8
-            mirror = 0 if  board.turn == chess.WHITE else 0x38
+            mirror = 0 if board.turn == chess.WHITE else 0x38
             bside = 0
 
         if not self.has_pawns:
@@ -1156,7 +1154,7 @@ class DtzTable(Table):
             while i < self.num:
                 piece_type = pc[i] & 0x07
                 color = (pc[i] ^ cmirror) >> 3
-                bb = board.pieces_mask(piece_type, color)
+                bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
                 square = chess.bit_scan(bb)
                 while square != -1 and square is not None:
@@ -1176,7 +1174,7 @@ class DtzTable(Table):
             k = self.files[0].pieces[0] ^ cmirror
             piece_type = k & 0x07
             color = k >> 3
-            bb = board.pieces_mask(piece_type, color)
+            bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
             i = 0
             p = [0, 0, 0, 0, 0, 0]
@@ -1193,7 +1191,7 @@ class DtzTable(Table):
             while i < self.num:
                 piece_type = pc[i] & 0x07
                 color = (pc[i] ^ cmirror) >> 3
-                bb = board.pieces_mask(piece_type, color)
+                bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
                 square = chess.bit_scan(bb)
                 while square != -1 and square is not None:
@@ -1238,7 +1236,8 @@ class Tablebases(object):
     Syzygy tables come in files like *KQvKN.rtbw* or *KRBvK.rtbz*, one WDL
     (*.rtbw*) and DTZ (*.rtbz*) file for each material composition.
 
-    Directly loads tables from *directory*. See *open_directory*.
+    Directly loads tables from *directory*. See
+    :func:`~chess.syzygy.Tablebases.open_directory`.
     """
     def __init__(self, directory=None, load_wdl=True, load_dtz=True):
         self.wdl = {}
@@ -1330,17 +1329,17 @@ class Tablebases(object):
         Probing is thread-safe when done with different *board* objects and
         if *board* objects are not modified during probing.
 
-        Returns *None* if the position was not found in any of the loaded
+        Returns ``None`` if the position was not found in any of the loaded
         tables.
 
-        Returns *2* if the side to move is winning, *0* if the position is
-        a draw and *-2* if the side to move is losing.
+        Returns ``2`` if the side to move is winning, ``0`` if the position is
+        a draw and ``-2`` if the side to move is losing.
 
-        Returns *1* in case of a cursed win and *-1* in case of a blessed
+        Returns ``1`` in case of a cursed win and ``-1`` in case of a blessed
         loss. Mate can be forced but the position can be drawn due to the
         fifty-move rule.
 
-        >>> with chess.syzygy.Tablebases("data/syzygy") as tablebases:
+        >>> with chess.syzygy.open_tablebases("data/syzygy") as tablebases:
         ...     tablebases.probe_wdl(chess.Board("8/2K5/4B3/3N4/8/8/4k3/8 b - - 0 1"))
         ...
         -2
@@ -1442,7 +1441,7 @@ class Tablebases(object):
             best = 0xffff
 
             for move in board.generate_legal_moves(pawns=False):
-                if board.piece_type_at(move.to_square) != chess.NONE:
+                if board.piece_type_at(move.to_square):
                     continue
 
                 board.push(move)
@@ -1497,21 +1496,22 @@ class Tablebases(object):
         Probing is thread-safe when done with different *board* objects and
         if *board* objects are not modified during probing.
 
-        Return *None* if the position was not found in any of the loaded tables.
-        Both DTZ and WDL tables are required in order to probe for DTZ values.
+        Return ``None`` if the position was not found in any of the loaded
+        tables. Both DTZ and WDL tables are required in order to probe for DTZ
+        values.
 
-        Returns a positive value if the side to move is winning, *0* if the
+        Returns a positive value if the side to move is winning, ``0`` if the
         position is a draw and a negative value if the side to move is losing.
 
         A non-zero distance to zero means the number of halfmoves until the
         next pawn move or capture can be forced, keeping a won position.
         Minmaxing the DTZ values guarantees winning a won position (and drawing
         a drawn position), because it makes progress keeping the win in hand.
-        However the lines are not always the most straight forward ways to win.
+        However the lines are not always the most straightforward ways to win.
         Engines like Stockfish calculate themselves, checking with DTZ, but only
         play according to DTZ if they can not manage on their own.
 
-        >>> with chess.syzygy.Tablebases("data/syzygy") as tablebases:
+        >>> with chess.syzygy.open_tablebases("data/syzygy") as tablebases:
         ...     tablebases.probe_dtz(chess.Board("8/2K5/4B3/3N4/8/8/4k3/8 b - - 0 1"))
         ...
         -53
@@ -1580,3 +1580,10 @@ class Tablebases(object):
 
     def __exit__(self, exc_type, exc_value, traceback):
         self.close()
+
+
+def open_tablebases(directory=None, load_wdl=True, load_dtz=True):
+    """
+    Opens a collection of tablebases for probing. See
+    :class:`~chess.syzygy.Tablebases`."""
+    return Tablebases(directory, load_wdl, load_dtz)
